(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/platform-browser')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/platform-browser'], factory) :
	(factory((global.ctrl = global.ctrl || {}, global.ctrl['ngx-csv'] = {}),global.ng.core,global.ng.platformBrowser));
}(this, (function (exports,core,platformBrowser) { 'use strict';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0
THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.
See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */










function __read(o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
}
function __spread() {
    for (var ar = [], i = 0; i < arguments.length; i++)
        ar = ar.concat(__read(arguments[i]));
    return ar;
}

var isJsons = function (array) { return Array.isArray(array) &&
    array.every(function (row) { return typeof row === 'object' && !(row instanceof Array); }); };
var isArrays = function (array) { return Array.isArray(array) && array.every(function (row) { return Array.isArray(row); }); };
function jsonsHeaders(array) {
    return Array.from(new Set(array.map(function (item) { return Object.keys(item); }).reduce(function (a, b) { return __spread(a, b); }, [])));
}
function jsons2arrays(jsons, headers) {
    headers = headers || jsonsHeaders(jsons);
    var headerLabels = (headers);
    var headerKeys = (headers);
    if (isJsons(headers)) {
        headerLabels = ((headers)).map(function (header) { return header.label; });
        headerKeys = ((headers)).map(function (header) { return header.key; });
    }
    var data = jsons.map(function (object) { return headerKeys.map(function (header) { return (header in object ? object[header] : ''); }); });
    return __spread([headerLabels], data);
}
var elementOrEmpty = function (element) { return element || element === 0 ? element : ''; };
function joiner(data, delimiter) {
    if (delimiter === void 0) { delimiter = ','; }
    return data
        .map(function (row, index) { return row.map(function (element) { return '"' + elementOrEmpty(element) + '"'; }).join(delimiter); })
        .join("\n");
}
function arrays2csv(data, headers, delimiter) {
    return joiner(headers ? __spread([headers], data) : data, delimiter);
}
function jsons2csv(data, headers, delimiter) {
    return joiner(jsons2arrays(data, headers), delimiter);
}
function string2csv(data, headers, delimiter) {
    return headers ? headers.join(delimiter) + "\n" + data : data;
}
function toCSV(data, headers, delimiter) {
    if (isJsons((data))) {
        return jsons2csv((data), headers, delimiter);
    }
    if (isArrays((data))) {
        return arrays2csv((data), headers, delimiter);
    }
    if (typeof data === 'string') {
        return string2csv(data, (headers), delimiter);
    }
    throw new TypeError("Data should be a \"String\", \"Array of arrays\" OR \"Array of objects\" ");
}
function buildURI(data, uFEFF, headers, delimiter) {
    if (uFEFF === void 0) { uFEFF = true; }
    var csv = toCSV(data, headers, delimiter);
    var blob = new Blob([uFEFF ? '\uFEFF' : '', csv], { type: 'text/csv' });
    return URL.createObjectURL(blob);
}
var CsvDirective = /** @class */ (function () {
    function CsvDirective(sanitizer) {
        this.sanitizer = sanitizer;
        this.data = [];
        this.delimiter = ',';
        this.uFEFF = true;
        this.download = 'data.csv';
        this.target = '_blank';
    }
    Object.defineProperty(CsvDirective.prototype, "filename", {
        set: function (a) {
            this.download = a;
        },
        enumerable: true,
        configurable: true
    });
    CsvDirective.prototype.ngOnChanges = function () {
        this.href = this.sanitizer.bypassSecurityTrustResourceUrl(buildURI(this.data, this.uFEFF, this.headers, this.delimiter));
    };
    return CsvDirective;
}());
CsvDirective.decorators = [
    { type: core.Directive, args: [{ selector: 'a[csvLink]' },] },
];
CsvDirective.ctorParameters = function () { return [
    { type: platformBrowser.DomSanitizer, },
]; };
CsvDirective.propDecorators = {
    "data": [{ type: core.Input },],
    "headers": [{ type: core.Input },],
    "delimiter": [{ type: core.Input },],
    "filename": [{ type: core.Input },],
    "uFEFF": [{ type: core.Input },],
    "href": [{ type: core.HostBinding },],
    "download": [{ type: core.HostBinding },],
    "target": [{ type: core.Input }, { type: core.HostBinding },],
};
var CsvModule = /** @class */ (function () {
    function CsvModule() {
    }
    return CsvModule;
}());
CsvModule.decorators = [
    { type: core.NgModule, args: [{
                declarations: [CsvDirective],
                exports: [CsvDirective],
            },] },
];
CsvModule.ctorParameters = function () { return []; };

exports.CsvModule = CsvModule;
exports.CsvDirective = CsvDirective;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ctrl-ngx-csv.umd.js.map
