import { __spread } from 'tslib';
import { Directive, HostBinding, Input, NgModule } from '@angular/core';
import { DomSanitizer } from '@angular/platform-browser';

var isJsons = function (array) { return Array.isArray(array) &&
    array.every(function (row) { return typeof row === 'object' && !(row instanceof Array); }); };
var isArrays = function (array) { return Array.isArray(array) && array.every(function (row) { return Array.isArray(row); }); };
function jsonsHeaders(array) {
    return Array.from(new Set(array.map(function (item) { return Object.keys(item); }).reduce(function (a, b) { return __spread(a, b); }, [])));
}
function jsons2arrays(jsons, headers) {
    headers = headers || jsonsHeaders(jsons);
    var headerLabels = (headers);
    var headerKeys = (headers);
    if (isJsons(headers)) {
        headerLabels = ((headers)).map(function (header) { return header.label; });
        headerKeys = ((headers)).map(function (header) { return header.key; });
    }
    var data = jsons.map(function (object) { return headerKeys.map(function (header) { return (header in object ? object[header] : ''); }); });
    return __spread([headerLabels], data);
}
var elementOrEmpty = function (element) { return element || element === 0 ? element : ''; };
function joiner(data, delimiter) {
    if (delimiter === void 0) { delimiter = ','; }
    return data
        .map(function (row, index) { return row.map(function (element) { return '"' + elementOrEmpty(element) + '"'; }).join(delimiter); })
        .join("\n");
}
function arrays2csv(data, headers, delimiter) {
    return joiner(headers ? __spread([headers], data) : data, delimiter);
}
function jsons2csv(data, headers, delimiter) {
    return joiner(jsons2arrays(data, headers), delimiter);
}
function string2csv(data, headers, delimiter) {
    return headers ? headers.join(delimiter) + "\n" + data : data;
}
function toCSV(data, headers, delimiter) {
    if (isJsons((data))) {
        return jsons2csv((data), headers, delimiter);
    }
    if (isArrays((data))) {
        return arrays2csv((data), headers, delimiter);
    }
    if (typeof data === 'string') {
        return string2csv(data, (headers), delimiter);
    }
    throw new TypeError("Data should be a \"String\", \"Array of arrays\" OR \"Array of objects\" ");
}
function buildURI(data, uFEFF, headers, delimiter) {
    if (uFEFF === void 0) { uFEFF = true; }
    var csv = toCSV(data, headers, delimiter);
    var blob = new Blob([uFEFF ? '\uFEFF' : '', csv], { type: 'text/csv' });
    return URL.createObjectURL(blob);
}
var CsvDirective = /** @class */ (function () {
    function CsvDirective(sanitizer) {
        this.sanitizer = sanitizer;
        this.data = [];
        this.delimiter = ',';
        this.uFEFF = true;
        this.download = 'data.csv';
        this.target = '_blank';
    }
    Object.defineProperty(CsvDirective.prototype, "filename", {
        set: function (a) {
            this.download = a;
        },
        enumerable: true,
        configurable: true
    });
    CsvDirective.prototype.ngOnChanges = function () {
        this.href = this.sanitizer.bypassSecurityTrustResourceUrl(buildURI(this.data, this.uFEFF, this.headers, this.delimiter));
    };
    return CsvDirective;
}());
CsvDirective.decorators = [
    { type: Directive, args: [{ selector: 'a[csvLink]' },] },
];
CsvDirective.ctorParameters = function () { return [
    { type: DomSanitizer, },
]; };
CsvDirective.propDecorators = {
    "data": [{ type: Input },],
    "headers": [{ type: Input },],
    "delimiter": [{ type: Input },],
    "filename": [{ type: Input },],
    "uFEFF": [{ type: Input },],
    "href": [{ type: HostBinding },],
    "download": [{ type: HostBinding },],
    "target": [{ type: Input }, { type: HostBinding },],
};
var CsvModule = /** @class */ (function () {
    function CsvModule() {
    }
    return CsvModule;
}());
CsvModule.decorators = [
    { type: NgModule, args: [{
                declarations: [CsvDirective],
                exports: [CsvDirective],
            },] },
];
CsvModule.ctorParameters = function () { return []; };

export { CsvModule, CsvDirective };
//# sourceMappingURL=ctrl-ngx-csv.js.map
